# swipeGuess

swipeGuess is a completion program intended to be used as a plugin for touchscreen-keyboards on mobile devices.

For each line input from stdin, it looks through a wordlist and outputs the first possible match for that gesture.

it's run like `input-program | swipeGuess wordlist.txt | output-program`

swipeGuess also provides options for returning multiple results and ignoring certain characters. See [the man page](https://git.sr.ht/~earboxer/swipeGuess/tree/master/item/swipeGuess.1.scd) for more information.

## input-program

The input program should output a stream of letters "swiped through", then a newline.

This is supported by [wvkbd](https://github.com/proycon/wvkbd) since version 0.6.

## wordlist

This should be a list of words, sorted by preferability of choice.

Good starting points can be found on the web, based on your language.

### Example English Word Lists

* http://www-personal.umich.edu/~jlawler/wordlist - 69,903 English words, including obscure words, missing common stems.
* http://www.mieliestronk.com/corncob_lowercase.txt - 58,110 English words with British spelling.
* https://raw.githubusercontent.com/dwyl/english-words/master/words.txt - 466,550 words, including many proper nouns like names.
* https://norvig.com/ngrams/count_1w.txt - 333,333 words, abbreviations, etc, sorted by popularity.
* https://www.keithv.com/software/wlist/index.php - page with several links providing intersections between english word lists (26,680 - 1,516,998 words)
* Your computer may have a list installed already in /usr/share/dict/, or one may be installable through your package manager.

### sorting tips

The most basic sorting technique would put longest words first, otherwise, it may be impossible to type them.

```
# Good autocompletion for terminal
ls $PATH | awk '{ print length, $0 }' | sort -nr | cut -d" " -f2- > binsSorted.txt

```

Smarter sorting would be keyboard-layout aware. mapScore can do that for you

```sh
./mapScore map.tsv <words.txt | sort -nr | cut -f2 > wordsSorted.txt
```

map.tsv uses tabs and newlines to create the grid-based layout. See `map.qwerty.noapos.tsv` for a sample of how to format this file.

If your keys are in a hexagonal layout, use mapScore like
`./mapScore map.simple.tsv bee`.

(`mapScore.py` was the old version of this, and probably should be removed, being slower and with less features.).

## output-program

* `completelyTypeWord.sh` (included in this repository) will type the characters (besides the first character, which is already typed by wvkbd) using wtype.

# Installation/Usage with wvkbd

1. Be using a wayland-based graphical shell (such as sway), and have wtype installed.
2. Compile with your favorite C compiler: `gcc swipeGuess.c -o swipeGuess`.
2. copy swipeGuess and completelyTypeWord.sh into your $PATH (`~/.local/bin/` or `/usr/local/bin/` for example)
3. `wvkbd-mobintl -O | swipeGuess /path/to/words.txt | completelyTypeWord.sh`
	* In SXMO, `KEYBOARD_ARGS='-O | swipeGuess /path/to/words.txt | completelyTypeWord.sh'` can be added to your ~/.profile to enable this (effective on restart).

# Extended information

[SwipeBehaviors](https://git.sr.ht/~earboxer/SwipeBehaviors) is a project that uses swipeGuess and provides more advanced functionality, like presenting several choices that can be picked with [suggpicker](https://git.sr.ht/~earboxer/suggpicker).
